import VueCompositionAPI, { defineComponent, provide, inject, ref } from '@vue/composition-api';
import mapboxgl, { Marker } from 'mapbox-gl';
import Deferred from 'my-deferred';
import Deferred$1 from 'my-deferred/dist/src';
import mixins from 'vue-typed-mixins';

var getMapboxOptions = function getMapboxOptions(props, el) {
  return {
    accessToken: props.accessToken,
    container: props.container || el,
    style: props.mapStyle
  };
};

var script = defineComponent({
  props: {
    accessToken: {
      type: String,
      default: ''
    },
    height: {
      type: String,
      default: '500px'
    },
    width: {
      type: String,
      default: '100%'
    },
    container: {
      type: [Object, String] || String,
      default: ''
    },
    minZoom: {
      type: Number,
      default: 0
    },
    maxZoom: {
      type: Number,
      default: 22
    },
    minPitch: {
      type: Number,
      default: 0
    },
    maxPitch: {
      type: Number,
      default: 60
    },
    mapStyle: {
      type: [Object, String],
      default: 'mapbox://styles/mapbox/streets-v11'
    },
    hash: {
      type: Boolean,
      default: false
    },
    interactive: {
      type: Boolean,
      default: true
    },
    bearingSnap: {
      type: Number,
      default: 7
    },
    pitchWithRotate: {
      type: Boolean,
      default: true
    },
    clickTolerance: {
      type: Number,
      default: 3
    },
    attributionControl: {
      type: Boolean,
      default: true
    },
    customAttribution: {
      type: Array,
      default: null
    },
    logoPosition: {
      type: String,
      default: 'bottom-left'
    },
    failIfMajorPerformanceCaveat: {
      type: Boolean,
      default: false
    },
    preserveDrawingBuffer: {
      type: Boolean,
      default: false
    },
    antialias: {
      type: Boolean,
      default: false
    },
    refreshExpiredTiles: {
      type: Boolean,
      default: true
    },
    maxBounds: {
      type: Array,
      default: undefined
    },
    scrollZoom: {
      type: [Boolean, Object],
      default: true
    },
    boxZoom: {
      type: Boolean,
      default: true
    },
    dragRotate: {
      type: Boolean,
      default: true
    },
    dragPan: {
      type: [Object, Boolean],
      default: true
    },
    keyboard: {
      type: Boolean,
      default: true
    },
    doubleClickZoom: {
      type: Boolean,
      default: true
    },
    touchZoomRotate: {
      type: [Boolean, Object],
      default: true
    },
    trackResize: {
      type: Boolean,
      default: true
    },
    center: {
      type: [Object, Array],
      default: function _default() {
        return [0, 0];
      }
    },
    zoom: {
      type: Number,
      default: 0
    },
    bearing: {
      type: Number,
      default: 0
    },
    pitch: {
      type: Number,
      default: 0
    },
    bounds: {
      type: Array,
      default: undefined
    },
    fitBoundsOptions: {
      type: Object,
      default: null
    },
    renderWorldCopies: {
      type: Boolean,
      default: true
    },
    maxTileCacheSize: {
      type: Number,
      default: null
    },
    localIdeographFontFamily: {
      type: String,
      default: 'sans-serif'
    },
    transformRequest: {
      type: Function,
      default: null
    },
    collectResourceTiming: {
      type: Boolean,
      default: false
    },
    fadeDuration: {
      type: Number,
      default: 300
    },
    crossSourceCollisions: {
      type: Boolean,
      default: true
    }
  },
  setup: function setup(props) {
    var $map = new Deferred();
    provide('$map', $map);
    return {
      $map: $map
    };
  },
  mounted: function mounted() {
    var _this = this;

    var options = getMapboxOptions(this.$props, this.$el);

    if (!options.accessToken) {
      throw new Error('No Access Token provided');
    }

    mapboxgl.accessToken = options.accessToken;
    var map = new mapboxgl.Map(options);
    map.on('load', function () {
      _this.$map.resolve(map);
    });
  },
  computed: {
    style: function style() {
      return {
        height: this.height,
        width: this.width
      };
    }
  }
});

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

/* script */
var __vue_script__ = script;
/* template */

var __vue_render__ = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    style: _vm.style
  }, [_vm.$map.isResolved ? _c('div', [_vm._t("default")], 2) : _c('div', [_vm._t("loader", [_c('div', {
    style: _vm.style
  })])], 2)]);
};

var __vue_staticRenderFns__ = [];
/* style */

var __vue_inject_styles__ = undefined;
/* scoped */

var __vue_scope_id__ = undefined;
/* module identifier */

var __vue_module_identifier__ = undefined;
/* functional template */

var __vue_is_functional_template__ = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__ = /*#__PURE__*/normalizeComponent({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__, __vue_module_identifier__, false, undefined, undefined, undefined);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var injectMap = {
  setup: function setup() {
    var $map = inject('$map');
    return {
      $map: $map
    };
  }
};

var getMarkerOptions = function getMarkerOptions(props) {
  var element = props.element,
      offset = props.offset,
      anchor = props.anchor,
      color = props.color,
      draggable = props.draggable,
      clickTolerance = props.clickTolerance,
      rotation = props.rotation,
      rotationAlignment = props.rotationAlignment,
      pitchAlignment = props.pitchAlignment,
      scale = props.scale;
  return {
    element: element,
    offset: offset,
    anchor: anchor,
    color: color,
    draggable: draggable,
    clickTolerance: clickTolerance,
    rotation: rotation,
    rotationAlignment: rotationAlignment,
    pitchAlignment: pitchAlignment,
    scale: scale
  };
};

var script$1 = defineComponent({
  props: {
    lngLat: {
      // type: Array as () => Array<number>,
      default: function _default() {
        return [0, 0];
      } // required: true,

    },
    element: {
      type: HTMLElement,
      default: null
    },
    offset: {
      type: [Array],
      default: function _default() {
        return [0, 0];
      }
    },
    anchor: {
      type: String,
      default: function _default() {
        return 'left';
      }
    },
    color: {
      type: String,
      default: null
    },
    scale: {
      type: Number,
      default: 1
    },
    draggable: {
      type: Boolean,
      default: false
    },
    rotation: {
      type: Number,
      default: 0
    },
    pitchAlignment: {
      type: String,
      default: 'auto'
    },
    rotationAlignment: {
      type: String,
      default: 'auto'
    }
  },
  setup: function setup() {
    var $map = injectMap.setup().$map;
    var $marker = new Deferred$1();
    provide('$marker', $marker);
    return {
      $map: $map,
      $marker: $marker
    };
  },
  mounted: function mounted() {
    return __awaiter(this, void 0, void 0, function () {
      var map, options, marker;
      return __generator(this, function (_a) {
        switch (_a.label) {
          case 0:
            return [4
            /*yield*/
            , this.$map.promise];

          case 1:
            map = _a.sent();
            options = getMarkerOptions(this.$props);
            marker = new Marker(options).setLngLat(this.lngLat).addTo(map);
            this.$marker.resolve(marker);
            return [2
            /*return*/
            ];
        }
      });
    });
  }
});

/* script */
var __vue_script__$1 = script$1;
/* template */

var __vue_render__$1 = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div');
};

var __vue_staticRenderFns__$1 = [];
/* style */

var __vue_inject_styles__$1 = undefined;
/* scoped */

var __vue_scope_id__$1 = undefined;
/* module identifier */

var __vue_module_identifier__$1 = undefined;
/* functional template */

var __vue_is_functional_template__$1 = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__$1 = /*#__PURE__*/normalizeComponent({
  render: __vue_render__$1,
  staticRenderFns: __vue_staticRenderFns__$1
}, __vue_inject_styles__$1, __vue_script__$1, __vue_scope_id__$1, __vue_is_functional_template__$1, __vue_module_identifier__$1, false, undefined, undefined, undefined);

// import { inject } from '../mixins/provide-inject.test';
var script$2 = {
  setup: function setup() {
    var stuff = inject('EP_FOO');
    var YoMomma = inject('YO_MOMMA');
    return {
      stuff: stuff,
      YoMomma: YoMomma
    };
  }
}; // export default {
//   inject: ['EP_FOO'],
//   computed: { stuff(this:any){ return this.EP_FOO; } }
// };

/* script */
var __vue_script__$2 = script$2;
/* template */

var __vue_render__$2 = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', [_vm._v(_vm._s(_vm.stuff))]);
};

var __vue_staticRenderFns__$2 = [];
/* style */

var __vue_inject_styles__$2 = undefined;
/* scoped */

var __vue_scope_id__$2 = undefined;
/* module identifier */

var __vue_module_identifier__$2 = undefined;
/* functional template */

var __vue_is_functional_template__$2 = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__$2 = /*#__PURE__*/normalizeComponent({
  render: __vue_render__$2,
  staticRenderFns: __vue_staticRenderFns__$2
}, __vue_inject_styles__$2, __vue_script__$2, __vue_scope_id__$2, __vue_is_functional_template__$2, __vue_module_identifier__$2, false, undefined, undefined, undefined);

var script$3 = {
  setup: function setup() {
    var EP_FOO = ref('1234');
    provide('EP_FOO', EP_FOO);
    provide('YO_MOMMA', 'IS FAT');
    var helloWorld = 15;
    return {
      EP_FOO: EP_FOO,
      helloWorld: helloWorld
    };
  }
};

/* script */
var __vue_script__$3 = script$3;
/* template */

var __vue_render__$3 = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', [_vm._t("default")], 2);
};

var __vue_staticRenderFns__$3 = [];
/* style */

var __vue_inject_styles__$3 = undefined;
/* scoped */

var __vue_scope_id__$3 = undefined;
/* module identifier */

var __vue_module_identifier__$3 = undefined;
/* functional template */

var __vue_is_functional_template__$3 = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__$3 = /*#__PURE__*/normalizeComponent({
  render: __vue_render__$3,
  staticRenderFns: __vue_staticRenderFns__$3
}, __vue_inject_styles__$3, __vue_script__$3, __vue_scope_id__$3, __vue_is_functional_template__$3, __vue_module_identifier__$3, false, undefined, undefined, undefined);

// import MapboxGeocoder from './MapboxGeocoder.vue';
var components = {
  // MapboxGeocoder,
  // MapboxImage,
  // MapboxImages,
  // MapboxLayer,
  MapboxMap: __vue_component__,
  MapboxMarker: __vue_component__$1,
  // MapboxNavigationControl,
  // MapboxPopup,
  // MapboxSource,
  // MapboxGeolocateControl,
  TestComponentInject: __vue_component__$2,
  TestComponentProvide: __vue_component__$3
};

var provideMap = mixins().extend({
  data: function data() {
    return {
      // VUE_MAPBOX_TS_static: Object.freeze({
      //   map: new Deferred() as Deferred<Map>,
      // }),
      id: '1234'
    };
  },
  provide: function provide() {
    return {
      // I_MAP: this.VUE_MAPBOX_TS_static.map,
      MY_ID: this.id
    };
  } // computed: {
  //   $map(){ return this.VUE_MAPBOX_TS_static.map; }
  // }

}); // export default mixins().extend({
//   data: () => ({
//     $static: Object.freeze({
//       map: new Deferred() as Deferred<Map>,
//     }),
//     somthing: ['Hello wolrd']
//   }),
//   provide(){
//     return {
//       $static: this.$static.map,
//     };
//   },
// });

var injectMap$1 = {
  inject: ['MY_ID'] // computed: {
  //   $map(){ return (this as any).I_MAP as Deferred<Map>; }
  // }

}; // export default mixins().extend({
//   // inject: [ '$map' ],
//   inject: ['todosLength'],
//   computed: {
//     map():Deferred<Map>{
//       return (this as any).$map as Deferred<Map>;
//     },
//   },
// });

/**
 * Install all components
 *
 * @param  {Vue}  Vue The Vue object
 * @return {void}
 */

function install(Vue) {
  // if(!Vue.__composition_api_installed__)
  Vue.use(VueCompositionAPI);
  Object.keys(components).forEach(function (name) {
    Vue.component(name, components[name]);
  });
}

export default install;
export { __vue_component__ as MapboxMap, __vue_component__$1 as MapboxMarker, __vue_component__$2 as TestComponentInject, __vue_component__$3 as TestComponentProvide, injectMap$1 as injectMap, install, provideMap };
//# sourceMappingURL=VueMapboxGl.esm.js.map
