/**
 * Map a mapbox element's events to the given vue element
 *
 * @param  {Vue}    vueElement    The Vue component in question
 * @param  {Mixed}  mapboxElement The Mapbox element bound to the component
 * @param  {Array}  events        The events to map
 * @param  {String} layerId       The layer on which the events are delegated
 * @return {Array}                The list of event/handler pair bounded
 */
function bindEvents(vueElement, mapboxElement, events, layerId) {
  if (events === void 0) {
    events = [];
  }

  if (layerId === void 0) {
    layerId = null;
  }

  var vueEvents = vueElement.$listeners; // eslint-disable-next-line no-param-reassign

  vueElement.$$events = Object.keys(vueEvents).reduce(function ($$events, vueEvent) {
    var originalEvent = vueEvent.replace(/^mb-/, '');

    if (!events.includes(originalEvent)) {
      return $$events;
    }

    var handler = function handler() {
      var payload = [];

      for (var _i = 0; _i < arguments.length; _i++) {
        payload[_i] = arguments[_i];
      }

      vueElement.$emit(vueEvent, payload);
    }; // If layerId is not null, all events must be
    // delegated from the map to the given layerId


    if (layerId) {
      mapboxElement.on(originalEvent, layerId, handler);
    } else {
      mapboxElement.on(originalEvent, handler);
    }

    $$events.push([originalEvent, handler]);
    return $$events;
  }, []);
}
/**
 * Unbind events from the map element
 *
 * @param  {Mixed} mapboxElement The Mapbox element which needs unbinding
 * @param  {Array}  handlers     The list of event/handler pair to unbind
 * @param  {String} layerId      The layer on which the events where delegated
 * @return {void}
 */

function unbindEvents(vueElement, mapboxElement, layerId) {
  if (layerId === void 0) {
    layerId = null;
  }

  vueElement.$$events.forEach(function (_a) {
    var event = _a[0],
        handler = _a[1]; // If layerId is not null, all events must be
    // delegated from the map to the given layerId

    if (layerId) {
      mapboxElement.off(event, layerId, handler);
    } else {
      mapboxElement.off(event, handler);
    }
  });
}

export { bindEvents, unbindEvents };
//# sourceMappingURL=bind-events.ts.map
